package com.company.jmix.test.screen.testentityview

import com.company.jmix.test.entity.TestEntity1
import com.company.jmix.test.entity.TestEntityView
import io.jmix.core.DataManager
import io.jmix.core.Id
import io.jmix.ui.ScreenBuilders
import io.jmix.ui.action.Action
import io.jmix.ui.component.GroupTable
import io.jmix.ui.model.CollectionLoader
import io.jmix.ui.screen.*
import org.springframework.beans.factory.annotation.Autowired

@UiController("jt_TestEntityView.browse")
@UiDescriptor("test-entity-view-browse.xml")
@LookupComponent("testEntityViewsTable")
class TestEntityViewBrowse : StandardLookup<TestEntityView>() {
    @Autowired
    private lateinit var screenBuilders: ScreenBuilders

    @Autowired
    private lateinit var testEntityViewsDl: CollectionLoader<TestEntityView>

    @Autowired
    private lateinit var testEntityViewsTable: GroupTable<TestEntityView>

    @Autowired
    private lateinit var dataManager: DataManager

    @Subscribe("testEntityViewsTable.create")
    private fun onTestEntityViewsTableCreate(event: Action.ActionPerformedEvent) {
        val screen = screenBuilders.editor(TestEntity1::class.java, this)
            .newEntity()
            .build()
        screen.addAfterCloseListener {
            testEntityViewsDl.load()
        }
        screen.show()
    }

    @Subscribe("testEntityViewsTable.edit")
    private fun onTestEntityViewsTableEdit(event: Action.ActionPerformedEvent) {
        val testEntityView = testEntityViewsTable.singleSelected ?: return
        val testEntity1 = dataManager.load(TestEntity1::class.java).id(testEntityView.id).one()

        val screen = screenBuilders.editor(TestEntity1::class.java, this)
            .editEntity(testEntity1)
            .build()
        screen.addAfterCloseListener {
            testEntityViewsDl.load()
        }
        screen.show()
    }

    @Subscribe("testEntityViewsTable.remove")
    private fun onTestEntityViewsTableRemove(event: Action.ActionPerformedEvent) {
        // todo conform dialog
        val testEntityView = testEntityViewsTable.singleSelected ?: return
        dataManager.remove(Id.of(testEntityView.id, TestEntity1::class.java))
        testEntityViewsDl.load()
    }


}